import argparse
from .stitch import stitch_template
import os
from .stitch_ipynb import stitch_notebook
import json

"""
Arguments to take :-
	name :- Name of the starter file to produce
			( .py or .ipynb extension only )

	domain : (Optional) Problem domain in dl,
			so to give appropriate models and data.
			
			Example
			vision : Conv Model and CIFAR-10 DATASET
			nlp : Conv Model and IMDB Sentiment Analysis DATASET
			structure : DNN with Iris Datset

	model - Model to pre-construct
	dataset - Tensorflow dataset to use
	training - choose Training loop ,custom(default) or fit
	test : Whether to include test loop, true(default) or false
	preprocessing : Added by default for selected models
"""

def main():

	# Important features 
	parser = argparse.ArgumentParser(prog="tf-stitch",
									description="Starter code with best practices for different Deep Learning problems in one command.",
									epilog="Thanks for Using.")

	# Argument for file to be generated
	parser.add_argument('output_file',
						type=str,
						help="the output file with starter code"
						)

	# Argument
	parser.add_argument('-d',
						'--domain',
						type=str,
						help="domain problem type for selecting "+
							  "appropriate model and dataset"
						)

	parser.add_argument('--dataset',
						type=str,
						help="select any of Tensorflow datasets",
						required=False
						)

	parser.add_argument('--model',
						type=str,
						help="select the Deep Learning Model",
						required=False
						)

	parser.add_argument('--training',
						type=str,
						default="custom",
						help="type of training loop, Custom or built-in",
						required=False
						)

	parser.add_argument('--testing',
						type=bool,
						default=True,
						help="if to include testing",
						required=False
						)

	# Create a Namespace object of all arguments
	args = parser.parse_args()

	# Extension of the file name provided
	output_extension = os.path.splitext( args.output_file )[-1]

	if not (output_extension in ['.py','.ipynb']):
		raise( Exception( "Output File extension must be .py or .ipynb " ) )

	# Domain provided in argument
	domain = args.domain

	dataset = None
	model = None

	# Make sure either domain is provided,
	# or the choice of dataset and model
	if domain is None:

		if args.dataset==None or args.model==None :
			raise( Exception( "Either provide domain eg. -d = vision \n"+
				"or provide dataset and model eg. tf-stitch cifar-10 conv" ) )
		else :

			dataset = args.dataset
			model = args.model


	training = args.training	
	testing = args.testing	

	# Get template codes into `content_blocks`
	content_blocks = stitch_template( domain ,
									  dataset,
									  model,
									  training,
									  testing )

	# Write the file using `content_blocks`
	with open( args.output_file , "w" ) as file:

		if output_extension==".ipynb":

			notebook_json = stitch_notebook( content_blocks )
			json.dump( notebook_json , file )

		else:

			divide_columns = 80
			# Section divider for ".py" file to separate out code blocks
			section_divider = "#"+"-"*(divide_columns)
			# Two newlines between each code blocks and section divider
			file.write( ("\n\n"+section_divider+"\n\n").join( content_blocks ) )

		file.close()

	# Show in message at end of program.
	success_msg = "Your code is stitched to your fit. Thanks for Using :)"
	print( success_msg )

if __name__=='__main__':
	main()







# It "stitch" together a simple jupyter notebook.
# Note , currently it only account for code blocks,
# as templates don't contanin any markdown , and
# vital information are provided as form of comments

def stitch_notebook( temp_blocks ):

    # `temp_blocks` consist of all template parts,
    # stored as a list

    # to store list of `cell` json
    cells = []

    for block in temp_blocks:
      
        cell_type = 'code'

        cell = {
            'cell_type': cell_type,
            'metadata': {},
            'source': block.splitlines(True),
            'outputs': [], 
            'execution_count': None
        }

        cells.append(cell)

    # It is standard way to write starter json of the notebook,
    # all constructed cell json goes into `cells` key
    notebook = {
        'nbformat': 4,
        'nbformat_minor': 0,
        'metadata': {
            'anaconda-cloud': {},
            'kernelspec': {
                'display_name': 'Python 3',
                'language': 'python',
                'name': 'python3'},
            'language_info': {
                'codemirror_mode': {'name': 'ipython', 'version': 3},
                'file_extension': '.py',
                'mimetype': 'text/x-python',
                'name': 'python',
                'nbconvert_exporter': 'python',
                'pygments_lexer': 'ipython3',
                'version': '3.6.1'}
        },
        'cells': cells,
    }

    return notebook





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tf-stitch", 
    version="1.2.1",
    author="Hemant Singh",
    keywords=["Deep learning" , "tensorflow" , "tf2.0" ,"keras",
             "Boilerplate" , "Starter Code" , "Starter Pieces" , "Quick Work" , "Productivity"],
    description="Quick Starter Code with different specifications stitched together in tensorflow 2.0 .",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/amifunny/tf-stitch/",
    maintainer="amifunny",
    entry_points={
        'console_scripts':[
            'tf-stitch=tf_stitch.__main__:main'
        ]
    },
    include_package_data=True,
    packages=setuptools.find_packages(),
    package_data={
        'tf_stitch':['templates/*/*.py','template.json']
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]

)


import json
import os

# path of where package is installed in system
RES_PATH = os.path.dirname(__file__)+"\\"

# Exception class for producing message , if argument
# given by user is invalid and also showing valid possible 
# options.
class ArgumentError(Exception):

	def __init__(self, argument , options):
		# Argument that is invalid
		self.argument = argument
		# Valid options from read json files
		self.options = options
		super().__init__(self.argument,self.options)

	def __str__(self):
		error_msg = "Invalid argument for `{}`\n".format(self.argument)
		option_msg = "Available options -> [{}]".format( " ,".join(self.options) )
		# Combining both messages
		return error_msg+option_msg


def get_files(**kwargs):

	# 'template.json' file contains mapping of arguments,
	# to possible values and their corresponding template filenames
	with open(RES_PATH+'template.json',"rb") as file:
		json_dict = json.loads( file.read() )
		file.close()

	# To hold different parts of required template,
	# as key-value pair,
	# For `template_files['model_file]`="conv_model.py"
	template_files = {}


	# Conditions check if argument is one of the keys,
	# in json of that argument type
	# For eg. 'domain' argument given by user, should be one
    # of ['conv','rnn' .. etc] that are keys in .json file
    # under the 'domain' field.
    # See `template.json` for reference


	if not (kwargs['domain'] in json_dict['domain'].keys()) and not kwargs['domain']==None:
		raise( ArgumentError("domain",json_dict['domain'].keys()) )

	if kwargs['domain']!=None :
		kwargs['dataset'] = json_dict['domain'][kwargs['domain']]['dataset']
		kwargs['model'] = json_dict['domain'][kwargs['domain']]['model']


	# Prompt to ask user to view valid tensorflow dataset names,
	# by browsing the `cata_link`
	cata_link = 'https://www.tensorflow.org/datasets/catalog/overview'
	print("Using dataset : {}. Check official catalogue at {}".format(kwargs['dataset'],cata_link))

	# Currently there are not varing options for 
	# import and dataset part of template
	template_files['import_file'] = json_dict['imports']
	template_files['dataset_file'] = json_dict['dataset']


	if not (kwargs['model'] in json_dict['model'].keys()):
		raise( ArgumentError("model",json_dict['domain'].keys()) )

	template_files['model_file'] = json_dict['model'][kwargs['model']]

	# Preprocessing option for few model. See "template.json".
	# Check if 'preprocessing' has key corresponding to current model.
	# If exist add to files dictionary.
	if kwargs['model'] in json_dict['preprocessing'].keys():
		template_files['preprocessing_file'] = json_dict['preprocessing'][kwargs['model']]

	if not (kwargs['training'] in json_dict['training'].keys()):
		raise( ArgumentError("training",json_dict['domain'].keys() ) )

	template_files['training_file'] = json_dict['training'][kwargs['training']]

	# add template of testing if True
	if kwargs['testing']:
		template_files['testing_file'] = json_dict['testing']

	# Also return datset , that was either given or
	# determined by domain
	return template_files,kwargs['dataset']

def stitch_template( domain=None,
					 dataset=None,
					 model=None,
					 training=None,
					 testing=None
					  ):

	# Get the template files name according to user's argument
	files,dataset = get_files( domain=domain,
					   dataset=dataset,
					   model=model,
					   training=training,
					   testing=testing )

	# Store code blocks, read from template files
	template_content = []

	with open( RES_PATH+'templates/imports/'+files['import_file'] , "r" ) as file:
		template_content.append( file.read().strip() )
		file.close()

	# For dataset template part , a special keyword => '%DATASET_NAME%',
	# is used to replace it with provided or default dataset name.
	with open( RES_PATH+'templates/dataset/'+files['dataset_file'] , "r" ) as file:
		template_content.append( file.read().strip().replace('%DATASET_NAME%',dataset) )
		file.close()

	if 'preprocessing_file' in files:
		with open( RES_PATH+'templates/preprocessing/'+files['preprocessing_file'] , "r" ) as file:
			template_content.append( file.read().strip() )
			file.close()

	with open( RES_PATH+'templates/model/'+files['model_file'] , "r" ) as file:
		template_content.append( file.read().strip() )
		file.close()	

	with open( RES_PATH+'templates/training/'+files['training_file'] , "r" ) as file:
		template_content.append( file.read().strip() )
		file.close()

	if files['testing_file']!=None:
		with open( RES_PATH+'templates/testing/'+files['testing_file'] , "r" ) as file:
			template_content.append( file.read().strip() )
			file.close()

	return template_content
